@@ -1,4 +1,5 @@
 /*  MikMod example player
+        (c) 2003 Raphael Assenat -- see file AUTHORS for complete list.
 	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
 	complete list.
 
@@ -20,7 +21,7 @@
 
 /*==============================================================================
 
-  $Id: display.c,v 1.38 1999/07/05 04:00:02 miod Exp $
+  $Id: display.c,v 1.12 2003/10/08 02:08:48 raph Exp $
 
   Display routines for the different panels and the playlist menu
 
@@ -57,6 +58,7 @@
 #include "mmenu.h"
 #include "mdialog.h"
 #include "mconfedit.h"
+#include "attr.h"
 
 /*========== Display layout */
 
@@ -83,11 +85,21 @@ typedef struct {
 
 extern BOOL quiet,semiquiet;
 extern MODULE *mf;
-
 static int cur_display=DISPLAY_SAMPLE,old_display=DISPLAY_SAMPLE;
 
 /* first line of displayed information in the panels */
 static int first_help=0,first_sample=0,first_inst=0,first_comment=0,first_list=0;
+static BOOL remove_msg=0;
+static time_t start_time;
+static char old_message[121];
+
+/* ================ Prototypes */
+
+void set_term_title(char *songname);
+
+
+
+
 
 /* computes printf templates when screen size changes, so that two-column
    display fills the screen */
@@ -109,9 +121,7 @@ static void setup_printf(void)
 #endif
 }
 
-static BOOL remove_msg=0;
-static time_t start_time;
-static char old_message[121];
+	
 
 /* displays a warning message on the top right corner of the display */
 void display_message(char *str)
@@ -194,6 +204,11 @@ static int display_banner(char* banner,BOOL root,int origin,int skip,BOOL wrap)
 
 	/* display lines */
 	for (i=skip;i<t;i++) {
+		if (i%2) 
+			attr_banner();
+		else
+			attr_banner1();
+		
 		for (n=0;(((n<winx)&&(n<MAXWIDTH))||(!wrap))&&
 		        (*buf!='\r')&&(*buf!='\n')&&(*buf);buf++) {
 			if (*buf<' ') str[n]=' ';
@@ -215,6 +230,7 @@ static int display_banner(char* banner,BOOL root,int origin,int skip,BOOL wrap)
 		else
 			win_clrtoeol(0,i-skip+origin);
 	}
+	attr_normal();
 
 	return skip;
 }
@@ -223,7 +239,9 @@ static int display_banner(char* banner,BOOL root,int origin,int skip,BOOL wrap)
 static void display_version(void)
 {
 	if (quiet) return;
+	attr_version();
 	win_print_root(0,0,mikversion);
+	attr_normal();
 }
 
 /* displays the "paused" banner */
@@ -253,8 +271,16 @@ void display_loadbanner(void)
 static void display_driver(void)
 {
 	char reverb[13];
+	int res;
 	
 	if (quiet) return;
+	
+	res = 0;
+
+	attr_driver1();
+	win_print_root(res,1,"Driver: ");
+	res += strlen("Driver: ");
+	
 	if (md_reverb)
 #ifdef HAVE_SNPRINTF
 		snprintf(reverb,12,"reverb: %2d",md_reverb);
@@ -273,80 +299,144 @@ static void display_driver(void)
 #ifdef HAVE_SNPRINTF
 	                STORAGELEN,
 #endif		 
-	    "%s: %d bit %s %s, %u Hz, %s\n",
+	    "%s, %d bit %s %s, %u Hz, %s\n",
 		md_driver->Name,(md_mode&DMODE_16BITS)?16:8,
 		(md_mode&DMODE_INTERP)?
 			(md_mode&DMODE_SURROUND?"interp. surround":"interpolated"):
 			(md_mode&DMODE_SURROUND?"surround":"normal"),
 		(md_mode&DMODE_STEREO)?"stereo":"mono",md_mixfreq,reverb);
 
-	win_print_root(0,1,storage);
+	attr_driver2();
+	win_print_root(res,1,storage);
+	attr_normal();
 }
 
 /* third line : filename */
 static void display_file(void)
 {
 	PLAYENTRY *entry;
+	int pos;
 
 	if (quiet) return;
 	if ((entry=PL_GetCurrent(&playlist))) {
 		CHAR *archive=entry->archive,*file;
 
-		if (archive) {
+		if (archive && !config.fullpaths) {
 			archive=strrchr(entry->archive,PATH_SEP);
-			if (archive) {
-				if (config.fullpaths)
-					while ((archive>entry->archive)&&(*--archive!=PATH_SEP));
-				else
-					archive++;
-			} else
+			if (archive)
+				archive++;
+			else
 				archive=entry->archive;
 		}
 
 		file=strrchr(entry->file,PATH_SEP);
-		if (file) {
-			if (config.fullpaths)
-				while ((file>entry->file)&&(*--file!=PATH_SEP));
-			else
-				file++;
-		} else
+		if (file && !config.fullpaths)
+			file++;
+		else
 			file=entry->file;
 
+		attr_file1();
+		win_print_root(0,2,"File: ");
+		attr_normal();
+
+		
+#ifdef HAVE_SNPRINTF
+		snprintf(storage,STORAGELEN,"%.70s\n",file);
+#else
+		sprintf(storage,"%.70s\n",file);
+#endif
+		attr_file2();
+		win_print_root(6,2,storage);
+		attr_normal();
+		pos = 6 + strlen(storage); // calculate where the archive name should start
+	
+		/* If the file is part of an archive, append the name inside () */
 		if ((archive)&&(strlen(file)<MAXWIDTH-13)) {
 			if (strlen(archive)+strlen(file)>MAXWIDTH-10) {
 				archive+=strlen(archive)-(MAXWIDTH-13-strlen(file));
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,"File: %s (...%s)\n",file,archive);
+				snprintf(storage,STORAGELEN,"(...%s)\n",archive);
 #else
-				sprintf(storage,"File: %s (...%s)\n",file,archive);
+				sprintf(storage,"(...%s)\n",archive);
 #endif
 			} else
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,"File: %s (%s)\n",file,archive);
+				snprintf(storage,STORAGELEN,"(%s)\n",archive);
 #else
-				sprintf(storage,"File: %s (%s)\n",file,archive);
+				sprintf(storage,"(%s)\n",archive);
 #endif
-		} else
-#ifdef HAVE_SNPRINTF
-			snprintf(storage,STORAGELEN,"File: %.70s\n",file);
-#else
-			sprintf(storage,"File: %.70s\n",file);
-#endif
-		win_print_root(0,2,storage);
+			attr_file3();	
+			win_print_root(pos,2,storage);
+			attr_normal();
+		} 			
 	}
 }
 
 /* fourth and fifth lines : module name and format */
 static void display_name(void)
 {
+	int pos=0;
+
 	if ((quiet)||(!mf)) return;
 
+	set_term_title(mf->songname);
+	
+	attr_song_name1();
+	win_print_root(0,3,"Name: ");
+	attr_normal();
 #ifdef HAVE_SNPRINTF
-	snprintf(storage,STORAGELEN,"Name: %.70s\n",mf->songname);
+	snprintf(storage,STORAGELEN,"%.70s\n",mf->songname);
 #else
-	sprintf(storage,"Name: %.70s\n",mf->songname);
+	sprintf(storage,"%.70s\n",mf->songname);
 #endif
-	win_print_root(0,3,storage);
+	attr_song_name2();
+	win_print_root(6,3,storage);
+	attr_normal();
+
+	/* Type label */
+	attr_type1();
+	win_print_root(0,4,"Type: ");
+	pos += 6;
+	attr_normal();
+
+	/* The type field */
+	attr_type2();
+	win_print_root(pos, 4, mf->modtype);
+	pos += strlen(mf->modtype);
+	win_print_root(pos, 4, ", ");
+	pos += 2;
+	attr_normal();
+
+	/* Period label */
+	attr_type3();
+	win_print_root(pos, 4, "Periods: ");
+	pos += strlen("Periods: ");
+	attr_normal();
+
+	/* The Period field */
+	attr_type4();
+	if (mf->flags&UF_XMPERIODS) {
+		win_print_root(pos, 4, "XM type, ");
+		pos += strlen("XM type, ");		
+	} else {
+		win_print_root(pos, 4, "mod type, ");
+		pos += strlen("mod type, ");
+	}
+	attr_normal();
+
+	/* The linear/log field */
+	attr_type5();
+	if (mf->flags&UF_LINEAR) {
+		win_print_root(pos, 4, "linear");
+		pos += strlen("linear");
+	} else {
+		win_print_root(pos, 4, "log");
+		pos += 3;
+	}
+	attr_normal();
+
+#if 0
+	
 #ifdef HAVE_SNPRINTF
 	snprintf
 #else
@@ -361,11 +451,13 @@ static void display_name(void)
 		(mf->flags&UF_XMPERIODS)?"XM type":"mod type",
 		(mf->flags&UF_LINEAR)?"linear":"log");
 	win_print_root(0,4,storage);
+#endif
 }
 
 /* sixth line : player status */
 void display_status(void)
 {
+	int pos;
 	if (quiet) return;
 
 	if (win_check_resize()) {
@@ -380,7 +472,8 @@ void display_status(void)
 		char time[7]="";
 		char channels[17]="";
 
-		if (cur && cur->time>0)
+		/* Build time string */
+		if (cur && cur->time>0) {
 #ifdef HAVE_SNPRINTF
 			snprintf(time,7,"/%2d:%02d",
 			        (int)((cur->time/60)%60),(int)(cur->time%60));
@@ -388,6 +481,10 @@ void display_status(void)
 			sprintf(time,"/%2d:%02d",
 			        (int)((cur->time/60)%60),(int)(cur->time%60));
 #endif
+		}
+			
+		/* Build Channels string */
+#if LIBMIKMOD_VERSION >= 0x030107
 		if (mf->flags&UF_NNA) {
 #ifdef HAVE_SNPRINTF
 			snprintf(channels,17,"%2d/%d+%d->%d",
@@ -396,7 +493,9 @@ void display_status(void)
 			sprintf(channels,"%2d/%d+%d->%d",
 			     mf->realchn,mf->numchn,mf->totalchn-mf->realchn,mf->totalchn);
 #endif
-		} else {
+		} else
+#endif
+		       {
 #ifdef HAVE_SNPRINTF
 			snprintf(channels,17,"%2d/%d      ",mf->realchn,mf->numchn);
 #else
@@ -404,6 +503,109 @@ void display_status(void)
 #endif
 		}
 
+		/* Print stuff to screen */
+		pos = 0;
+		
+		/* pattern label */
+		attr_status_line1();
+		win_print_root(pos, 5, "pat:");
+		pos += 4;
+		
+		/* pattern field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%03d/%03d ",mf->sngpos,mf->numpos-1);
+#else
+		sprintf(storage, "%03d/%03d ",mf->sngpos,mf->numpos-1);
+#endif
+		attr_status_line2();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+
+		/* pos label */
+		attr_status_line3();
+		win_print_root(pos, 5, "pos:");
+		pos += 4;
+		
+		/* pos field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%02.2X ",mf->patpos);
+#else
+		sprintf(storage, "%02.2X ",mf->patpos);
+#endif
+		attr_status_line4();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+		
+		/* spd label */
+		attr_status_line5();
+		win_print_root(pos, 5, "spd:");
+		pos += 4;
+		
+		/* spd field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%2d/%3d ",mf->sngspd,mf->bpm);
+#else
+		sprintf(storage, "%2d/%3d ",mf->sngspd,mf->bpm);
+#endif
+		attr_status_line6();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+		
+		/* vol label */
+		attr_status_line7();
+		win_print_root(pos, 5, "vol:");
+		pos += 4;
+		
+		/* vol field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%3d%%/%3d%% ",
+						(mf->volume*100+64)>>7, (md_volume*100+64)>>7);
+#else
+		sprintf(storage, "%3d%%/%3d%% ",
+						(mf->volume*100+64)>>7, (md_volume*100+64)>>7);
+#endif
+		attr_status_line8();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+		
+		/* time label */
+		attr_status_line9();
+		win_print_root(pos, 5, "time:");
+		pos += 5;
+		
+		/* time field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%2d:%02d%s ",
+						(int)(((mf->sngtime>>10)/60)%60),
+						(int)((mf->sngtime>>10)%60),
+						time);
+#else
+		sprintf(storage, "%2d:%02d%s ",
+						(int)(((mf->sngtime>>10)/60)%60),
+						(int)((mf->sngtime>>10)%60),
+						time);
+#endif
+		attr_status_line10();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+		
+		/* chn label */
+		attr_status_line11();
+		win_print_root(pos, 5, "chn:");
+		pos += 4;
+		
+		/* chn field */
+#ifdef HAVE_SNPRINTF
+		snprintf(storage, STORAGELEN, "%s  \n", channels);
+#else
+		sprintf(storage, STORAGELEN, "%s  \n", channels);
+#endif
+		attr_status_line12();
+		win_print_root(pos, 5, storage);
+		pos += strlen(storage);
+	
+		attr_normal();
+#if 0
 #ifdef HAVE_SNPRINTF
 		snprintf
 #else
@@ -421,6 +623,7 @@ void display_status(void)
 			time,channels);
 
 		win_print_root(0,5,storage);
+#endif
 
 	}
 }
@@ -465,7 +668,8 @@ static void convert_string(char *str)
 static void display_sample(int diff)
 {
 	int count,semicount,t,x,winx;
-
+	char smp_name[20], smp_num[10];
+	
 	first_sample+=diff;
 	win_get_size(&winx,&semicount);
 	if (winx<MINWIDTH+MINVISIBLE) count=semicount;
@@ -482,6 +686,9 @@ static void display_sample(int diff)
 		else count=semicount*2;
 	}
 
+	//snprintf(fmt_halfwidth,20,"%%3i %%-%d.%ds",halfwidth-4,halfwidth-4);
+	snprintf(smp_name,20,"%%-%d.%ds",halfwidth-4,halfwidth-4);
+	
 	for (t=first_sample;t<mf->numsmp&&t<(count+first_sample);t++) {
 		x=((t-first_sample)<semicount)?0:halfwidth;
 		if (x<winx) {
@@ -494,10 +701,18 @@ static void display_sample(int diff)
 #ifdef HAVE_SNPRINTF
 			                STORAGELEN,
 #endif
-			fmt_halfwidth,t,
+			smp_name,
 			mf->samples[t].samplename?mf->samples[t].samplename:"");
 			convert_string(storage);
-			win_print(x,(t-first_sample)%semicount,storage);
+
+			snprintf(smp_num, 10, "%3i ", t);
+			attr_sample_num();
+			win_print(x,(t-first_sample)%semicount,smp_num);
+	
+			attr_sample_name();
+			win_print(x+4,(t-first_sample)%semicount,storage);
+			attr_normal();
+			
 		}
 	}
 }
@@ -506,6 +721,7 @@ static void display_sample(int diff)
 static void display_inst(int diff)
 {
 	int count,semicount,t,x,winx;
+	char inst_name[20], inst_num[10];
 
 	first_inst+=diff;
 	win_get_size(&winx,&semicount);
@@ -523,6 +739,8 @@ static void display_inst(int diff)
 		else count=semicount*2;
 	}
 
+	snprintf(inst_name,20,"%%-%d.%ds",halfwidth-4,halfwidth-4);
+	
 	for (t=first_inst;t<mf->numins&&t<(count+first_inst);t++) {
 		x=((t-first_inst)<semicount)?0:halfwidth;
 		if (x<winx) {
@@ -535,12 +753,21 @@ static void display_inst(int diff)
 #ifdef HAVE_SNPRINTF
 			                STORAGELEN,
 #endif
-			fmt_halfwidth,t,
+			inst_name,
 			mf->instruments[t].insname ? mf->instruments[t].insname:"");
 			convert_string(storage);
-			win_print(x,(t-first_inst)%semicount,storage);
+		
+			
+			snprintf(inst_num, 10, "%3i ", t);
+			attr_inst_num();
+			win_print(x,(t-first_sample)%semicount,inst_num);
+			
+			attr_inst_name();
+			win_print(x+4,(t-first_inst)%semicount,storage);
+			
 		}
 	}
+	attr_normal();
 }
 
 /* comment panel */
@@ -829,89 +1056,127 @@ static void display_playentry(PLAYENTRY *pos,PLAYENTRY *cur,int nr,int y,int x,B
 		timelen=strlen(time);
 	}
 	name=strrchr(pos->file,PATH_SEP);
-	if (name) {
-		if (config.fullpaths)
-			while ((name>pos->file)&&(*--name!=PATH_SEP));
-		else
-			name++;
-	} else
+	if (name && !config.fullpaths)
+		name++;
+	else
 		name=pos->file;
 
 	if (pos==cur) sort='>';
 	else if (pos->played) sort='*';
 	else sort=' ';
 
+	/* Display number */
+	attr_list_num();
+#ifdef HAVE_SNPRINTF
+	snprintf(storage, STORAGELEN, "%4i ", nr);
+#else
+	snprintf(storage, "%4i ", nr);
+#endif
+	win_print(x,y,storage);
+	x += strlen(storage);
+	attr_normal();
+
+	
+	/* Display sort */
+	attr_list_sort();
+#ifdef HAVE_SNPRINTF
+	snprintf(storage, STORAGELEN, "%c", sort);
+#else
+	snprintf(storage, "%c", sort);
+#endif
+	win_print(x,y,storage);
+	x += strlen(storage);
+	attr_normal();
+
+	
 	if (pos->archive) {
 		if (strlen(name)>width-13-timelen) {
 			name=name+strlen(name)-(width-16-timelen);
 			if (timelen) {
-				sprintf(tmpfmt,"%%4i %%c...%%-%ds%%s(pack)",width-22);
+				sprintf(tmpfmt,"...%%-%ds%%s",width-22);
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name,time);
+				snprintf(storage,STORAGELEN,tmpfmt,name,time);
 #else
-				sprintf(storage,tmpfmt,nr,sort,name,time);
+				sprintf(storage,tmpfmt,name,time);
 #endif
 			} else {
-				sprintf(tmpfmt,"%%4i %%c...%%-%ds(pack)",width-16);
+				sprintf(tmpfmt,"...%%-%ds",width-16);
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name);
+				snprintf(storage,STORAGELEN,tmpfmt,name);
 #else
-				sprintf(storage,tmpfmt,nr,sort,name);
+				sprintf(storage,tmpfmt,name);
 #endif
 			}
-		} else
+
+		} else {
 			if (timelen) {
-				sprintf(tmpfmt,"%%4i %%c%%-%ds%%s(pack)",width-19);
+				sprintf(tmpfmt,"%%-%ds%%s",width-19);
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name,time);
+				snprintf(storage,STORAGELEN,tmpfmt,name,time);
 #else
-				sprintf(storage,tmpfmt,nr,sort,name,time);
+				sprintf(storage,tmpfmt,name,time);
 #endif
 			} else {
-				sprintf(tmpfmt,"%%4i %%c%%-%ds(pack)",width-13);
+				sprintf(tmpfmt,"%%-%ds",width-13);
 #ifdef HAVE_SNPRINTF
-				snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name);
+				snprintf(storage,STORAGELEN,tmpfmt,name);
 #else
-				sprintf(storage,tmpfmt,nr,sort,name);
+				sprintf(storage,tmpfmt,name);
 #endif
 			}
-	} else if (strlen(name)>33-timelen) {
-		name=name+strlen(name)-(30-timelen);
+		}
+	
+		
+	} 
+	else if (strlen(name)>width-7-timelen) {
+		name=name+strlen(name)-(width-10-timelen);
 		if (timelen) {
-			sprintf(tmpfmt,"%%4i %%c...%%-%ds%%s",width-16);
+			sprintf(tmpfmt,"...%%-%ds%%s",width-16);
 #ifdef HAVE_SNPRINTF
-			snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name,time);
+			snprintf(storage,STORAGELEN,tmpfmt,name,time);
 #else
-			sprintf(storage,tmpfmt,nr,sort,name,time);
+			sprintf(storage,tmpfmt,name,time);
 #endif
 		} else {
-			sprintf(tmpfmt,"%%4i %%c...%%-%ds",width-10);
+			sprintf(tmpfmt,"...%%-%ds",width-10);
 #ifdef HAVE_SNPRINTF
-			snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name);
+			snprintf(storage,STORAGELEN,tmpfmt,name);
 #else
-			sprintf(storage,tmpfmt,nr,sort,name);
+			sprintf(storage,tmpfmt,name);
 #endif
 		}
-	} else
+	} else {
 		if (timelen) {
-			sprintf(tmpfmt,"%%4i %%c%%-%ds%%s",width-13);
+			sprintf(tmpfmt,"%%-%ds%%s",width-13);
 #ifdef HAVE_SNPRINTF
-			snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name,time);
+			snprintf(storage,STORAGELEN,tmpfmt,name,time);
 #else
-			sprintf(storage,tmpfmt,nr,sort,name,time);
+			sprintf(storage,tmpfmt,name,time);
 #endif
 		} else {
-			sprintf(tmpfmt,"%%4i %%c%%-%ds",width-7);
+			sprintf(tmpfmt,"%%-%ds",width-7);
 #ifdef HAVE_SNPRINTF
-			snprintf(storage,STORAGELEN,tmpfmt,nr,sort,name);
+			snprintf(storage,STORAGELEN,tmpfmt,name);
 #else
-			sprintf(storage,tmpfmt,nr,sort,name);
+			sprintf(storage,tmpfmt,name);
 #endif
 		}
+	}
 
 	if (reverse) win_attrset(A_REVERSE);
+	attr_list_name();
 	win_print(x,y,storage);
+	x+=strlen(storage);
 	win_attrset(A_NORMAL);
+	attr_normal();
+	
+	if (pos->archive) {
+		attr_list_pack();
+		win_print(x,y,"(pack)");
+		attr_normal();
+	}
+	
+//	win_print(x,y,storage);
 }
 
 /* playlist panel */
@@ -1018,6 +1283,7 @@ static BOOL display_information(int diff,COMMAND com)
 {
 	char paneltitle[STORAGELEN],otherpanels[STORAGELEN];
 	BOOL change=0;
+	int pos=0;
 
 	if (quiet || semiquiet) return 1;
 
@@ -1074,27 +1340,73 @@ static BOOL display_information(int diff,COMMAND com)
 			break;
 	}
 	
+	pos = 0;
+	attr_info1();
+	win_print_root(pos, 6, paneltitle);
+	pos += strlen(paneltitle);
+	
+	attr_info2();
+	win_print_root(pos, 6, " --");
+	pos += 3;
+		
 	/* set available panel information */
 	strcpy(otherpanels," --");
 
-	if (cur_display!=DISPLAY_HELP)
-		strcat(otherpanels," H:[help]");
-	if ((cur_display!=DISPLAY_INST) && mf && (mf->flags&UF_INST))
-		strcat(otherpanels," I:[instruments]");
-	if (cur_display!=DISPLAY_SAMPLE && mf)
-		strcat(otherpanels," S:[samples]");
-	if ((cur_display!=DISPLAY_COMMENT) && mf && (mf->comment))
-		strcat(otherpanels," M:[message]");
-	if (cur_display!=DISPLAY_LIST)
-		strcat(otherpanels," L:[playlist]");
-	if (cur_display!=DISPLAY_CONFIG)
-		strcat(otherpanels," C:[config]");
-	strcat (otherpanels,"\n");
+	if (cur_display!=DISPLAY_HELP) {
+		//strcat(otherpanels," H:[help]");
+		attr_info3();
+		win_print_root(pos, 6, " H:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[help]");
+		pos += 6;
+	}
+	if ((cur_display!=DISPLAY_INST) && mf && (mf->flags&UF_INST)) {
+		//strcat(otherpanels," I:[instruments]");
+		attr_info3();
+		win_print_root(pos, 6, " I:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[instruments]");
+		pos += 13;
+	}
+	if (cur_display!=DISPLAY_SAMPLE && mf) {
+		//strcat(otherpanels," S:[samples]");
+		attr_info3();
+		win_print_root(pos, 6, " S:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[samples]");
+		pos += 9;
+	}
+	if ((cur_display!=DISPLAY_COMMENT) && mf && (mf->comment)) {
+		//strcat(otherpanels," M:[message]");
+		attr_info3();
+		win_print_root(pos, 6, " M:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[messages]");
+		pos += 10;
+	}
+	if (cur_display!=DISPLAY_LIST) {
+		//strcat(otherpanels," L:[playlist]");
+		attr_info3();
+		win_print_root(pos, 6, " L:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[playlist]");
+		pos += 10;
+	}
+	if (cur_display!=DISPLAY_CONFIG) {
+		//strcat(otherpanels," C:[config]");
+		attr_info3();
+		win_print_root(pos, 6, " C:"); pos += 3;
+		attr_info4();
+		win_print_root(pos, 6, "[config]");
+		pos += 8;
+	}
+	attr_normal();
+	//strcat (otherpanels,"\n");
 
-	win_attrset(A_REVERSE);
-	win_print_root(0,6,paneltitle);
-	win_attrset(A_NORMAL);
-	win_print_root(strlen(paneltitle),6,otherpanels);
+//	win_attrset(A_REVERSE);
+//	win_print_root(0,6,paneltitle);
+//	win_attrset(A_NORMAL);
+//	win_print_root(strlen(paneltitle),6,otherpanels);
 
 	/* display panel contents */
 	switch(cur_display) {
@@ -1219,6 +1531,75 @@ void display_init(void)
 	}
 	win_change_panel(cur_display);
 	setup_printf();
+
+	
+}
+
+/* This will set the xterm (or equivalent) Title and Icon title.
+ *
+ * The title contains -= MikMod x.x.x =- (%s) where %s is the song name
+ * the icon contains -= MikMod x.x.x =-
+ *
+ * pass NULL as songname to reset the title
+ * 
+ * Written using the 'Xterm-Title mini-howto' */
+void set_term_title(char *songname)
+{	
+	char *env_term;
+	static int last_config=0;
+
+	if (!config.xterm_title && !last_config) { 
+		return; 
+	}
+	if (last_config && !config.xterm_title) {
+		/* xterm title setting has just been disabled */
+		songname = NULL;
+	}
+
+	last_config = config.xterm_title;
+	
+	env_term = getenv("TERM");
+	if (env_term==NULL) { return; }
+
+	if (songname!=NULL)
+	{
+#ifdef HAVE_SNPRINTF
+		snprintf(storage,STORAGELEN,"%s (%s)", mikversion, songname);
+#else
+		sprintf(storage,"%s: (%s)", mikversion, songname);
+#endif
+	}
+	else
+	{
+#ifdef HAVE_SNPRINTF
+		snprintf(storage,STORAGELEN,"");
+#else
+		sprintf(storage,"");
+#endif
+	}
+
+	if ( strcmp(env_term, "xterm")==0 ||
+		strcmp(env_term, "xterm-color")==0 ||
+		strcmp(env_term, "rxvt")==0 ||
+		strcmp(env_term, "aixterm")==0 ||
+		strcmp(env_term, "dtterm")==0 ||
+		strcmp(env_term, "xterm-color")==0 ||
+		strcmp(env_term, "xterm-color")==0 ||
+		strcmp(env_term, "xterm-color")==0 ) 
+	{	
+		printf("%c]0;%s%c", '\033', storage, '\007');
+		printf("%c]1;%s%c", '\033', mikversion, '\007');
+	} 
+	else if (strcmp(env_term, "iris-ansi")==0) 
+	{
+		printf("%cP1.y%s%c\\", '\033', storage, '\033');	
+		printf("%cP3.y%s%c\\", '\033', mikversion, '\033');	
+	} 
+	else if (strcmp(env_term, "hpterm")==0) 
+	{
+		printf("\033&f0k%dD%s", strlen(storage), storage);
+		printf("\033&f-1k%dD%s", strlen(mikversion), mikversion);
+	} 
 }
 
 /* ex:set ts=4: */
