@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mikmod.c,v 1.49 1999/07/05 04:00:02 miod Exp $
+  $Id: mikmod.c,v 1.11 2003/10/08 02:08:48 raph Exp $
 
   Module player example of MikMod
 
@@ -70,6 +70,7 @@ void usleep(unsigned long);
 #include "player.h"
 #include "mwindow.h"
 #include "mdialog.h"
+#include "attr.h"
 
 #if defined(__FreeBSD__)||defined(__NetBSD__)||defined(__OpenBSD__)
 #ifdef HAVE_SYS_TIME_H
@@ -111,6 +112,8 @@ static struct option options[]={
 	{"nopanning",		no_argument,		NULL,  8},
 	{"protracker",		no_argument,		NULL,'x'},
 	{"noprotracker",	no_argument,		NULL,  9},
+	{"exitafter",		no_argument,		NULL,'X'},
+	{"noexitafter",		no_argument,		NULL, 15},
 	/* Loading options */
 	{"curious",			no_argument,		NULL,'c'},
 	{"nocurious",		no_argument,		NULL, 10},
@@ -125,6 +128,10 @@ static struct option options[]={
 	/* Display options */
 	{"quiet",			no_argument,		NULL,'q'},
 	{"terse",			no_argument,		NULL, 14},
+#ifdef ENABLE_COLOR_INTERFACE
+	{"color",			no_argument,		NULL,'C'},
+	{"nocolor",			no_argument,		NULL, 16},
+#endif
 	/* Information options */
 	{"information",		no_argument,		NULL,'n'},
 	{"version",			no_argument,		NULL,'V'},
@@ -177,59 +184,74 @@ static void help(CONFIG *c)
 		"Usage: %s [option]... [module|playlist]...\n"
 		"\n"
 		"Output options:\n"
-		"  -d[river] n,options     Use nth driver for output (0: autodetect), default: %d\n"
-		"  -o[utput] 8m|8s|16m|16s 8/16 bit output in stereo/mono, default: %s\n"
-		"  -f[requency] nnnnn      Set mixing frequency, default: %d\n"
-		"* -i[nterpolate]          Use interpolate mixing, default: %s\n"
-		"* -hq[mixer]              Use high-quality (but slower) software mixer,\n"
-		"                          default: %s\n"
-		"* -su[rround]             Use surround mixing, default: %s\n"
-		"  -r[everb] nn            Set reverb amount (0-15), default: %d\n"
+		"  -d, --driver n,options     Use nth driver for output (0: autodetect),\n"
+		"                                default: %d\n"
+		"  -o, --output 8m|8s|16m|16s 8/16 bit output in stereo/mono, default: %s\n"
+		"  -f, --frequency nnnnn      Set mixing frequency, default: %d\n"
+		"* -i, --interpolate          Use interpolate mixing, default: %s\n"
+		"*     --hqmixer              Use high-quality (but slower) software mixer,\n"
+		"                                default: %s\n"
+		"*     --surround             Use surround mixing, default: %s\n"
+		"  -r, --reverb nn            Set reverb amount (0-15), default: %d\n"
 		"Playback options:\n"
-		"  -v[olume] nn            Set volume from 0%% (silence) to 100%%, default: %d%%\n"
-		"* -F, -fa[deout]          Force volume fade at the end of module, default: %s\n"
-		"* -l[oops]                Enable in-module loops, default: %s\n"
-		"* -a, -pa[nning]          Process panning effects, default: %s\n"
-		"* -x, -pr[otracker]       Disable extended protracker effects, default: %s\n"
+		"  -v, --volume nn            Set volume from 0%% (silence) to 100%%, default: %d%%\n"
+		"* -F, --fadeout              Force volume fade at the end of module,\n"
+		"                                default: %s\n"
+		"* -l, --loops                Enable in-module loops, default: %s\n"
+		"* -a, --panning              Process panning effects, default: %s\n"
+		"* -x, --protracker           Disable extended protracker effects, default: %s\n"
+		"* -X, --exitafter            Exit MikMod when the playlist is finished,\n"
+		"                                default: %s\n"
 		"Loading options:\n"
-		"* -c[urious]              Look for hidden patterns in module, default: %s\n"
-		"  -p[laymode] n           Playlist mode (1: loop module, 2: list multi\n"
-		"                             4: shuffle list, 8: list random), default: %d\n"
-		"* -t[olerant]             Don't halt on file access errors, default: %s\n"
+		"* -c, --curious              Look for hidden patterns in module, default: %s\n"
+		"  -p, --playmode n           Playlist mode (1: loop module, 2: list multi,\n"
+		"                                4: shuffle list, 8: list random), default: %d\n"
+		"* -t, --tolerant             Don't halt on file access errors, default: %s\n",
+		PRG_NAME,c->driver,output,c->frequency,S_B(c->interpolate),
+		S_B(c->hqmixer),S_B(c->surround),c->reverb,c->volume,S_B(c->fade),
+		S_B(c->loop),S_B(c->panning),S_B(!c->extspd),S_B(c->exit_after),
+		S_B(c->curious),c->playmode,S_B(c->tolerant)
+			);
+
 #if defined(__OS2__)||defined(__EMX__)||defined(__linux)||defined(__FreeBSD__)||defined(__NetBSD__)||defined(__OpenBSD__)
 
 #if defined(__OS2__)||defined(__EMX__)
-		"Scheduling options:\n"
+	puts("Scheduling options:");
 #else
-		"Scheduling options (need root privileges or a setuid root binary):\n"
+	puts("Scheduling options (need root privileges or a setuid root binary):");
 #endif
-		"* -s, -ren[ice]           Renice to -20 (more scheduling priority), default: %s\n"
+	printf(
+		"* -s, --renice               Renice to -20 (more scheduling priority),\n"
+		"                                default: %s\n",
+		(c->renice==RENICE_PRI?"Yes":"No"));
 #if !defined(__NetBSD__)&&!defined(__OpenBSD__)
-		"* -S, -rea[ltime]         Get realtime priority (will hog CPU power), default: %s\n"
+	printf(
+		"* -S, --realtime             Get realtime priority (will hog CPU power),\n"
+		"                                default: %s\n",
+		(c->renice==RENICE_REAL?"Yes":"No"));
 #endif
 
 #endif
+
+	printf(
 		"Display options:\n"
-		"  -q[uiet]                Quiet mode, no interface, displays only errors.\n"
-		"  -te[rse]                Terse output : status line only.\n"
-		"Information options:\n"
-		"  -n, -inf[ormation]      List all available drivers and module loaders.\n"
-		"  -V -ve[rsion]           Display MikMod version.\n"
-		"  -h[elp]                 Display this help screen.\n"
-		"Configuration option:\n"
-		"  -norc                   Don't parse the file '%s' on startup\n"
-		"\n"
-        "Options marked with '*' also exist in negative form (eg -nointerpolate)\n"
-		"F1 or H while playing: Display help panel.\n",
-		PRG_NAME,c->driver,output,c->frequency,S_B(c->interpolate),
-		S_B(c->hqmixer),S_B(c->surround),c->reverb,c->volume,S_B(c->fade),
-		S_B(c->loop),S_B(c->panning),S_B(!c->extspd),S_B(c->curious),
-		c->playmode,S_B(c->tolerant),
-#if defined(__OS2__)||defined(__EMX__)||defined(__linux)||defined(__FreeBSD__)||defined(__NetBSD__)||defined(__OpenBSD__)
-		(c->renice==RENICE_PRI?"Yes":"No"),
-#if !defined(__NetBSD__)&&!defined(__OpenBSD__)
-		(c->renice==RENICE_REAL?"Yes":"No"),
+		"  -q, --quiet                Quiet mode, no interface, displays only errors.\n"
+		"      --terse                Terse output : status line only.\n"
+#ifdef ENABLE_COLOR_INTERFACE
+		"  -C, --color                Enable color interface. default: %s\n"
 #endif
+		"Information options:\n"
+		"  -n, --information          List all available drivers and module loaders.\n"
+		"  -V, --version              Display MikMod version.\n"
+		"  -h, --help                 Display this help screen.\n"
+		"Configuration option:\n"
+		"      --norc                 Don't parse the file '%s'\n"
+		"                                on startup\n"
+		"\n"
+        "Options marked with '*' also exist in negative form (eg --nointerpolate)\n"
+		"F1 or H while playing: Display help panel.\n",
+#ifdef ENABLE_COLOR_INTERFACE
+		S_B(c->color),	
 #endif
 		conf_name);
 
@@ -286,6 +308,8 @@ static void exit_player(int status,char *message,...)
 		vprintf(message,args);
 	va_end(args);
 
+	set_term_title(NULL); // reset the title
+	
 	exit(status);
 }
 
@@ -321,6 +345,7 @@ static RETSIGTYPE ExitGracefully(int signum)
 
 		signal(SIGINT,SIG_DFL);
 		signal(SIGTERM,SIG_DFL);
+		set_term_title(NULL); // reset the title
 		exit(0);
 	}
 }
@@ -404,12 +429,17 @@ void Player_SetConfig(CONFIG *cfg)
 	             ((cfg->driver)&&(cfg->driver!=md_device))||
 	             (!cmp_bit(md_mode,DMODE_16BITS,cfg->mode_16bit))||
 	             (!cmp_bit(md_mode,DMODE_STEREO,cfg->stereo))||
-	             (!cmp_bit(md_mode,DMODE_HQMIXER,cfg->hqmixer))||
-				 ((!driveroptions && cfg->driveroptions) ||
-				  (driveroptions && strcmp(driveroptions,cfg->driveroptions))));
+	             (!cmp_bit(md_mode,DMODE_HQMIXER,cfg->hqmixer))
+#if LIBMIKMOD_VERSION >= 0x030107
+				 ||((!driveroptions && cfg->driveroptions) ||
+				  (driveroptions && strcmp(driveroptions,cfg->driveroptions)))
+#endif
+	             );
 
+#if LIBMIKMOD_VERSION >= 0x030107
 	if (driveroptions) free(driveroptions);
 	driveroptions=strdup(cfg->driveroptions);
+#endif
 
 	md_pansep=128; /* panning separation (0=mono 128=full stereo) */
 	md_volume=(cfg->volume*128)/100;
@@ -552,7 +582,11 @@ int main(int argc,char *argv[])
 	/* Parse commandline */
 	opterr=0;
 	while ((t=getopt_long_only(argc,argv,
-			   "d:o:f:r:v:p:i:F:l:a:x:c:t:s:S:qnVh",options,NULL))
+			   "d:o:f:ir:v:Flaxcp:tsSqnVhX"
+#ifdef ENABLE_COLOR_INTERFACE
+			   "C"
+#endif
+			   ,options,NULL))
 #if defined(__OS2__)||defined(__EMX__)
 	      >=0
 #else
@@ -636,6 +670,10 @@ int main(int argc,char *argv[])
 				config.extspd=0; break;
 			case   9: /* --noprotracker */
 				config.extspd=1; break;
+			case 'X': /* -X --exitafter */
+				config.exit_after=1; break;
+			case  15: /* --noexitafter */
+				config.exit_after=0; break;
 			case 'c': /* -c --curious */
 				config.curious=1; break;
 			case  10: /* --nocurious */
@@ -658,6 +696,12 @@ int main(int argc,char *argv[])
 				quiet=1; break;
 			case  14: /* --terse */
 				semiquiet=1; break;
+#ifdef ENABLE_COLOR_INTERFACE
+			case 'C': /* --color */
+				config.color=1; break;
+			case 16: /* --nocolor */
+				config.color=0; break;
+#endif
 			case 'n': /* -n --information */
 				puts(mikcopyr);
 				printf("Sound engine version %ld.%ld.%ld\n",
@@ -667,7 +711,7 @@ int main(int argc,char *argv[])
 				       "\nRecognized module formats are :\n%s\n",
 				       MikMod_InfoDriver(),MikMod_InfoLoader());
 				exit(0);
-			case 'V': /* --version */
+			case 'V': /* -V --version */
 				puts(mikcopyr);
 				printf("Sound engine version %ld.%ld.%ld\n",
 				       (engineversion>>16)&255,(engineversion>>8)&255,
@@ -714,6 +758,7 @@ int main(int argc,char *argv[])
 
 	/*  initialize interface */
 	win_init(quiet);
+	attr_init_colors(quiet);
 	display_init();
 
 	signal(SIGTERM,ExitGracefully);
@@ -934,6 +979,8 @@ int main(int argc,char *argv[])
 		if (config.save_config) CF_Save(&config);
 		if (config.save_playlist) PL_SaveDefault(&playlist);
 	}
+
+	set_term_title(NULL); // reset the title
 	exit(0);
 }
 
