@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mutilities.h,v 1.3 1999/07/05 04:00:02 miod Exp $
+  $Id: mutilities.h,v 1.1.1.1 2003/09/19 13:16:01 raph Exp $
 
   Some utility functions
 
@@ -37,6 +37,8 @@ char *str_sprintf(char*,char*);
 char *get_tmp_name(void);
 
 BOOL file_exist(char*);
+/* determines if a given path is absolute or relative */
+BOOL path_relative(char*);
 
 /* allocate and return a filename including the path for a config file
    'name': filename without the path */
