@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: mwindow.c,v 1.3 1999/07/05 04:00:02 miod Exp $
+  $Id: mwindow.c,v 1.2 2003/09/30 01:53:30 raph Exp $
 
   Some window functions
 
@@ -43,6 +43,7 @@
 #include <mikmod.h>
 #include "player.h"
 #include "mwindow.h"
+#include "attr.h"
 
 #define PANEL_HELP    1
 #define PANEL_SAMPLE  2
@@ -559,8 +560,10 @@ static void win_status_repaint(void)
 	int i;
 	if (win_quiet) return;
 
+	attr_bottom_status_line1();
 	if ((root_y2>1)&&(win->height>1))
 		for (i=0;i<win->width;i++) mvaddch(win->height-2,i,BOX_HLINE);
+	attr_normal();
 }
 
 /* init the status line(height=0,1,2  0: no status line) */
